"""Package of examples of stonerplots usage."""

__all__ = [
    # 1 Default Plots
    "default_plot",
    "default_plot_latex",
    # 2 Journal Styles
    "default_plot_ieee",
    "default_plot_aps",
    "default_plot_aip",
    "default_plot_iop",
    "default_plot_nature",
    "default_plot_aps_1_5_column",
    "default_plot_aps_2_column",
    "default_plot_thesis",
    "default_plot_science",
    # 3 Formats
    "scatter_plot",
    # 4 Colour Schemes
    "default_plot_std_colours",
    "default_plot_bright",
    "default_plot_high_contrast",
    "default_plot_high_vis",
    "default_plot_light",
    "default_plot_muted",
    "default_plot_retro",
    "default_plot_vibrant",
    "dark_plot",
    # 5 Formats
    "notebook_plot",
    "poster_plot",
    "presentation_plot",
    "small_presentation_plot",
    "small_presentation_dark_plot",
    "hi_res_plot",
    # 6 Misc
    "grid_plot",
    # 7 Context Managers
    "inset_plot",
    "stacked_plot",
    "multi_panel_plot",
    # Extra
    "xrr_example",
    "tri_plot",
    "penta_plot",
]
